from __future__ import annotations

from typing import Any, Dict, List

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class AKSUpgradeChannel(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure AKS cluster upgrade channel is chosen"
        id = "CKV_AZURE_171"
        supported_resources = ("azurerm_kubernetes_cluster",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
        )

    def scan_resource_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        if 'automatic_channel_upgrade' in conf:
            automatic_channel_upgrade = conf.get('automatic_channel_upgrade')
            if isinstance(automatic_channel_upgrade, list) and automatic_channel_upgrade != ['none']:
                return CheckResult.PASSED

        if 'automatic_upgrade_channel' in conf:
            automatic_upgrade_channel = conf.get('automatic_upgrade_channel')
            if isinstance(automatic_upgrade_channel, list) and automatic_upgrade_channel != ['none']:
                return CheckResult.PASSED

        return CheckResult.FAILED

    def get_evaluated_keys(self) -> List[str]:
        return ['automatic_upgrade_channel', 'automatic_channel_upgrade']


check = AKSUpgradeChannel()



# flake8: noqa
# type: ignore

import json
import os

from checkov.main import secrets_runner
from checkov.common.runners.runner_registry import RunnerRegistry
from checkov.runner_filter import RunnerFilter
from checkov.common.bridgecrew.platform_integration import bc_integration


with open(os.environ['LOCAL_SECRETS_POLICIES_JSON']) as secrets_policies_file:
    default_regexes = json.load(secrets_policies_file)
bc_integration.customer_run_config_response = {'secretsPolicies': default_regexes}


def execute():
    runner = secrets_runner(entropy_limit=4)
    # 20 min less in order to finish processing, else put checkov's default (12h - 1200)
    runner_registry = RunnerRegistry(
        '',
        RunnerFilter(
            block_list_secret_scan=[],
            enable_secret_scan_all_files=True,
            enable_git_history_secret_scan=False,
            git_history_last_commit_scanned=None,
            git_history_timeout="checkov_timeout_str",
            checks=['BC_GIT_79']
        ),
        runner
    )

    scan_reports = runner_registry.run(
        root_folder=os.environ["LOCAL_SCANNING_FOLDER"],
        external_checks_dir=list(),
        collect_skip_comments=True)

    print(scan_reports)


if __name__ == "__main__":
    execute()



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class EMRPubliclyAccessible(BaseResourceValueCheck):

    def __init__(self):
        name = "Ensure AWS EMR block public access setting is enabled"
        id = "CKV_AWS_390"
        supported_resources = ['aws_emr_block_public_access_configuration']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "block_public_security_group_rules"


check = EMRPubliclyAccessible()



from __future__ import annotations

import re
import typing

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck


STO_NAME_REGEX = re.compile(r"^[a-z0-9]{3,24}$")
VARIABLE_REFS = ("local.", "module.", "var.", "random_string.", "random_id.", "random_integer.", "random_pet.",
                 "azurecaf_name", "each.", "substring")


class StorageAccountName(BaseResourceCheck):
    def __init__(self) -> None:
        """
        Initializes a check to ensure that Storage Accounts adhere to the naming rules.

        The naming reference for Storage Accounts can be found here:
        https://docs.microsoft.com/en-us/azure/storage/common/storage-account-overview#naming-storage-accounts
        """
        name = "Ensure Storage Accounts adhere to the naming rules"
        id = "CKV_AZURE_43"
        supported_resources = ['Microsoft.Storage/storageAccounts']
        categories = [CheckCategories.CONVENTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, typing.Any]) -> CheckResult:
        """
        The Storage Account naming reference:
        https://docs.microsoft.com/en-us/azure/storage/common/storage-account-overview#naming-storage-accounts
        :param conf: azurerm_storage_account configuration
        :return: <CheckResult>
        """
        if "name" in conf.keys():
            name = conf["name"]
            if name:
                name = str(name)
                if any(x in name for x in VARIABLE_REFS):
                    # in the case we couldn't evaluate the name, just ignore
                    return CheckResult.UNKNOWN
                if re.findall(STO_NAME_REGEX, name):
                    return CheckResult.PASSED
        return CheckResult.FAILED

    def get_evaluated_keys(self) -> typing.List[str]:
        return ["name"]


check = StorageAccountName()



from __future__ import annotations

from typing import cast, Any

from checkov.common.parsers.node import StrNode, ListNode
from checkov.common.util.consts import START_LINE, END_LINE, LINE_FIELD_NAMES
from checkov.common.util.suppression import collect_suppressions_for_report
from checkov.serverless.utils import ServerlessElements


def build_definitions_context(definitions: dict[str, dict[str, Any]], definitions_raw: dict[str, list[tuple[int, str]]]
                              ) -> dict[str, dict[str, Any]]:
    definitions_context: dict[str, dict[str, Any]] = {}
    for file_path, file_definitions in definitions.items():
        definitions_context[file_path] = {}
        for definition_attribute, definition_value in file_definitions.items():
            if definition_attribute not in [str(e) for e in ServerlessElements]:
                continue
            definitions_context[file_path][definition_attribute] = {}
            if isinstance(definition_value, dict):
                for resource_key, resource_attributes in definition_value.items():
                    add_resource_to_definitions_context(definitions_context, resource_key, resource_attributes,
                                                        definition_attribute, definitions_raw, file_path)
            elif isinstance(definition_value, list):
                for resource in definition_value:
                    add_resource_to_definitions_context(definitions_context, '', resource, definition_attribute,
                                                        definitions_raw, file_path)

            elif isinstance(definition_value, StrNode):
                add_resource_to_definitions_context(definitions_context, definition_attribute, definition_value,
                                                    definition_attribute,
                                                    definitions_raw, file_path)
    return definitions_context


def add_resource_to_definitions_context(definitions_context: dict[str, dict[str, Any]], resource_key: str,
                                        resource_attributes: dict[str, Any] | ListNode | StrNode, definition_attribute: str,
                                        definitions_raw: dict[str, Any], file_path: str) -> None:
    if resource_key in LINE_FIELD_NAMES:
        return

    if resource_attributes:
        if isinstance(resource_attributes, dict) and START_LINE in resource_attributes and END_LINE in resource_attributes:
            start_line = resource_attributes[START_LINE] - 1
            end_line = resource_attributes[END_LINE] - 1
        elif isinstance(resource_attributes, ListNode):
            start_line = resource_attributes.start_mark.line
            end_line = resource_attributes.end_mark.line
        elif isinstance(resource_attributes, StrNode):
            start_line = resource_attributes.start_mark.line + 1
            end_line = resource_attributes.end_mark.line + 1
        else:
            return
    else:
        return

    definition_resource = {"start_line": start_line, "end_line": end_line}

    if resource_key is None and isinstance(resource_attributes, dict):
        resource_key = f"{resource_attributes.get('type')}.{resource_attributes.get('name')}"
    int_start_line = cast(int, definition_resource["start_line"])
    int_end_line = cast(int, definition_resource["end_line"])
    code_lines_for_suppressions_check = definitions_raw[file_path][int_start_line: int_end_line]
    definition_resource['skipped_checks'] = collect_suppressions_for_report(
        code_lines=code_lines_for_suppressions_check)
    if isinstance(resource_attributes, dict) and 'type' in resource_attributes:
        definition_resource["type"] = resource_attributes.get('type')

    definition_resource["code_lines"] = definitions_raw[file_path][start_line - 1: end_line]

    definitions_context[file_path][definition_attribute][resource_key] = definition_resource



from __future__ import annotations

import os
import logging
from typing import List, Tuple, Dict, Any
from checkov.common.typing import _SkippedCheck
from checkov.common.bridgecrew.integration_features.features.policy_metadata_integration import integration as metadata_integration


class ContextParser:
    """
    Context parser for secrets scanning — supports:
    - Metadata suppressions like:
      {
        "Metadata": {
          "checkov": {
            "skip": [
              {"id": "CKV_SECRET_6", "comment": "example reason"}
            ]
          }
        }
      }
    """

    def __init__(self, file_path: str) -> None:
        self.file_path = file_path
        self.file_lines: List[Tuple[int, str]] = self._read_file_lines()

    def _read_file_lines(self) -> List[Tuple[int, str]]:
        if not os.path.exists(self.file_path):
            return []
        with open(self.file_path, "r", encoding="utf-8") as f:
            return [(i + 1, line.rstrip('\n')) for i, line in enumerate(f.readlines())]

    def collect_skip_comments(
            self,
            resource_config: Dict[str, Any] | List[Dict[str, Any]] | None = None
    ) -> List[_SkippedCheck]:
        """
        Collects suppressions from resource metadata.

        Supports:
        - Metadata under 'checkov' and 'bridgecrew' keys
        - Object-rooted and array-rooted JSON/YAML structures

        Returns a list of suppression dicts
        """
        skipped_checks: List[_SkippedCheck] = []
        bc_id_mapping = metadata_integration.bc_to_ckv_id_mapping

        def extract_skips(metadata_block: Dict[str, Any]) -> None:
            for source in ("checkov", "bridgecrew"):
                for skip in metadata_block.get(source, {}).get("skip", []):
                    skip_id = skip.get("id")
                    skip_comment = skip.get("comment", "No comment provided")
                    if not skip_id:
                        logging.warning("Check suppression is missing key 'id'")
                        continue

                    skipped_check: _SkippedCheck = {
                        "id": skip_id,
                        "suppress_comment": skip_comment,
                    }

                    if bc_id_mapping and skip_id in bc_id_mapping:
                        skipped_check["bc_id"] = skip_id
                        skipped_check["id"] = bc_id_mapping[skip_id]
                    elif metadata_integration.check_metadata:
                        skipped_check["bc_id"] = metadata_integration.get_bc_id(skip_id)

                    skipped_checks.append(skipped_check)

        if isinstance(resource_config, dict):
            metadata = resource_config.get("Metadata", {})
            extract_skips(metadata)

        elif isinstance(resource_config, list):
            for item in resource_config:
                if isinstance(item, dict):
                    metadata = item.get("Metadata", {})
                    extract_skips(metadata)

        return skipped_checks



from typing import List, Any

from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck
from checkov.common.models.enums import CheckCategories


class KubernetesClusterHTTPApplicationRouting(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        name = "Ensure Azure AKS cluster HTTP application routing is disabled"
        id = "CKV_AZURE_246"
        supported_resources = ('azurerm_kubernetes_cluster',)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "http_application_routing_enabled"

    def get_forbidden_values(self) -> List[Any]:
        return [True]


check = KubernetesClusterHTTPApplicationRouting()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class S3GlobalViewACL(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure AWS S3 bucket does not have global view ACL permissions enabled"
        id = "CKV_AWS_375"
        supported_resource = ("aws_s3_bucket_acl",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resource)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        if 'access_control_policy' in conf:
            for policy_idx, policy in enumerate(conf["access_control_policy"]):
                if 'grant' in policy:
                    for grant_idx, grant in enumerate(policy["grant"]):
                        self.evaluated_keys = [f"access_control_policy/[{policy_idx}]/grant/[{grant_idx}]/permission"]
                        if (isinstance(grant, dict) and 'permission' in grant and
                                ('FULL_CONTROL' in grant.get('permission') or 'READ_ACP' in grant.get('permission'))):
                            if 'grantee' in grant:
                                for grantee in grant.get('grantee'):
                                    if 'uri' in grantee and 'http://acs.amazonaws.com/groups/global/AllUsers' in grantee.get('uri'):
                                        return CheckResult.FAILED

        return CheckResult.PASSED


check = S3GlobalViewACL()



from __future__ import annotations

from typing import Any, List

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck


class FunctionAppsAccessibleOverHttps(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure that Function apps is only accessible over HTTPS"
        id = "CKV_AZURE_70"
        supported_resources = (
            "Microsoft.Web/sites/config",
            "Microsoft.Web/sites",
            "Microsoft.Web/sites/slots",
        )
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        if "properties" in conf:
            if self.entity_type == "Microsoft.Web/sites" or self.entity_type == "Microsoft.Web/sites/slots":
                if "httpsOnly" not in conf["properties"]:
                    return CheckResult.FAILED

                https_only = conf["properties"]["httpsOnly"]
                if not https_only:
                    return CheckResult.FAILED

        if "httpSettings" in conf["properties"]:
            auth_settings_v2 = conf["properties"]["httpSettings"]

            # default=true for require_https
            if 'requireHttps' not in auth_settings_v2:
                return CheckResult.PASSED

            require_https = auth_settings_v2.get("requireHttps")
            if not require_https:
                return CheckResult.FAILED

        return CheckResult.PASSED

    def get_evaluated_keys(self) -> List[str]:
        return ["properties", "properties/httpsOnly", "properties/httpSettings"]


check = FunctionAppsAccessibleOverHttps()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.arm.base_resource_check import BaseResourceCheck


class APIManagementMinTLS12(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure API management uses at least TLS 1.2"
        id = "CKV_AZURE_173"
        supported_resources = ("Microsoft.ApiManagement/service",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        properties = conf.get("properties")
        if isinstance(properties, dict) and "customProperties" in properties:
            self.evaluated_keys = ["properties"]
            customProperties = properties.get("customProperties")
            if isinstance(customProperties, dict):
                self.evaluated_keys = ["properties/customProperties"]
                if customProperties.get("Microsoft.WindowsAzure.ApiManagement.Gateway.Security.Backend.Protocols.Ssl30"):
                    return CheckResult.FAILED
                if customProperties.get("Microsoft.WindowsAzure.ApiManagement.Gateway.Security.Backend.Protocols.Tls10"):
                    return CheckResult.FAILED
                if customProperties.get("Microsoft.WindowsAzure.ApiManagement.Gateway.Security.Protocols.Ssl30"):
                    return CheckResult.FAILED
                if customProperties.get("Microsoft.WindowsAzure.ApiManagement.Gateway.Security.Protocols.Tls10"):
                    return CheckResult.FAILED
                if customProperties.get("Microsoft.WindowsAzure.ApiManagement.Gateway.Security.Protocols.Tls11"):
                    return CheckResult.FAILED
        return CheckResult.PASSED


check = APIManagementMinTLS12()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class UnpatchedAuroraPostgresDB(BaseResourceCheck):

    def __init__(self):
        name = "Ensure AWS Aurora PostgreSQL is not exposed to local file read vulnerability"
        id = "CKV_AWS_388"
        supported_resources = ['aws_db_instance']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'engine' in conf and 'aurora-postgresql' in conf['engine']:
            if 'engine_version' in conf and conf['engine_version'][0] in ['10.11', '10.12', '10.13', '11.6', '11.7', '11.8']:
                self.evaluated_keys = ['engine', 'engine-version']
                return CheckResult.FAILED
        return CheckResult.PASSED


check = UnpatchedAuroraPostgresDB()



from __future__ import annotations

from pathlib import Path
from typing import Any

from checkov.serverless.parsers.parser import FUNCTIONS_TOKEN, PROVIDER_TOKEN, IAM_ROLE_STATEMENTS_TOKEN, \
    ENVIRONMENT_TOKEN, STACK_TAGS_TOKEN, TAGS_TOKEN
from checkov.cloudformation.context_parser import ContextParser as CfnContextParser, STARTLINE, ENDLINE
from checkov.common.util.file_utils import read_file_with_any_encoding


class ContextParser(object):
    """
    serverless functions template context parser
    """
    # control on inherited provider attributes to scanned functions
    # Values are the source and destination
    ENRICHED_ATTRIBUTES = [
        (IAM_ROLE_STATEMENTS_TOKEN, IAM_ROLE_STATEMENTS_TOKEN),
        (ENVIRONMENT_TOKEN, ENVIRONMENT_TOKEN),
        (STACK_TAGS_TOKEN, TAGS_TOKEN),
        ("runtime", "runtime"),
        ("timeout", "timeout"),
        ("memorySize", "memorySize")
    ]

    def __init__(self, sls_file: str, sls_template: dict[str, Any], sls_template_lines: list[tuple[int, str]]) -> None:
        self.sls_file = sls_file
        self.sls_template = sls_template
        self.sls_template_lines = sls_template_lines
        self.provider_conf = sls_template.get(PROVIDER_TOKEN)
        self.functions_conf = sls_template.get(FUNCTIONS_TOKEN) or {}
        self.provider_type = self._infer_provider_type()

    def file(self, content: dict[str, Any]) -> str:
        if isinstance(content, dict):
            return str(content.get('__file__', self.sls_file))
        return self.sls_file

    def extract_code_lines(
        self, content: dict[str, Any]
    ) -> tuple[list[int], list[tuple[int, str]]] | tuple[None, None]:
        find_lines_result_list = list(CfnContextParser.find_lines(content, STARTLINE))
        if len(find_lines_result_list) >= 1:
            start_line = min(find_lines_result_list) - 1
            end_line = max(list(CfnContextParser.find_lines(content, ENDLINE)))

            entity_lines_range = [start_line, end_line - 1]

            fname = self.file(content)
            lines = self.sls_template_lines
            if fname != self.sls_file:
                lines = []
                text = read_file_with_any_encoding(Path(self.sls_file).parent / fname)
                for i, ln in enumerate(text.splitlines(True)):
                    lines.append((i + 1, ln))
            entity_code_lines = lines[start_line - 1: end_line - 1]
            return entity_lines_range, entity_code_lines
        return None, None

    def enrich_function_with_provider(self, sls_function_name: str) -> None:
        """
        Update inplace a function's template with pre-defined inherited provider attributes
        :param sls_function_name: scanned function
        :return: None
        """
        if not self.provider_conf or not isinstance(self.provider_conf, dict):
            return

        for src_attribute, dst_enriched_attribute in self.ENRICHED_ATTRIBUTES:
            provider_attribute = self.provider_conf.get(src_attribute)
            if not provider_attribute:
                continue

            template_function = self.functions_conf.get(sls_function_name)
            if not template_function:
                continue
            function_attribute = template_function.get(dst_enriched_attribute)
            if function_attribute:
                if not isinstance(function_attribute, type(provider_attribute)):
                    # Do not enrich maps with strings etc
                    continue
                if isinstance(template_function[dst_enriched_attribute], list):
                    template_function[dst_enriched_attribute].extend(provider_attribute)
                if isinstance(template_function[dst_enriched_attribute], dict):
                    template_function[dst_enriched_attribute].update(provider_attribute)
            else:
                template_function[dst_enriched_attribute] = provider_attribute

    def _infer_provider_type(self) -> str | None:
        if isinstance(self.provider_conf, dict):
            return self.provider_conf.get('name')
        if isinstance(self.provider_conf, str):
            return self.provider_conf

        return None



from __future__ import annotations

from typing import List, Dict, Any, Union, TYPE_CHECKING

from checkov.terraform.checks.data.base_cloudsplaining_data_iam_check import BaseTerraformCloudsplainingDataIAMCheck

if TYPE_CHECKING:
    from cloudsplaining.scan.policy_document import PolicyDocument


class CloudSplainingPrivilegeEscalation(BaseTerraformCloudsplainingDataIAMCheck):
    def __init__(self) -> None:
        name = "Ensure IAM policies does not allow privilege escalation"
        id = "CKV_AWS_110"
        super().__init__(name=name, id=id)

    def cloudsplaining_analysis(self, policy: PolicyDocument) -> Union[List[str], List[Dict[str, Any]]]:
        escalations = policy.allows_privilege_escalation
        flattened_escalations: list[str] = []
        if escalations:
            for escalation in escalations:
                if isinstance(escalation, dict):
                    flattened_escalations.extend(escalation.get('actions'))
                else:
                    flattened_escalations.append(escalation)
        return flattened_escalations


check = CloudSplainingPrivilegeEscalation()



from typing import List

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import \
    BaseResourceCheck


class VPCSecurityGroupRuleSet(BaseResourceCheck):
    def __init__(self):
        name = "Ensure Tencent Cloud VPC security group rules do not accept all traffic"
        id = "CKV_TC_8"
        supported_resources = ['tencentcloud_security_group_rule_set']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> CheckResult:
        if conf.get("ingress"):
            for i in conf["ingress"]:
                if i.get("action") and i["action"][0] != "ACCEPT":
                    continue
                if i.get("cidr_block") is None and i.get("ipv6_cidr_block") is None:
                    continue
                if i.get("cidr_block") and i["cidr_block"][0] != "0.0.0.0/0":
                    continue
                if i.get("ipv6_cidr_block") and (i["ipv6_cidr_block"][0] not in ["::/0", "0::0/0"]):
                    continue
                return CheckResult.FAILED

        return CheckResult.PASSED

    def get_evaluated_keys(self) -> List[str]:
        return ["ingress"]


check = VPCSecurityGroupRuleSet()



from __future__ import annotations

import json
import logging
import os
import re
from pathlib import Path
from typing import List, Callable, TYPE_CHECKING, Any, Optional, Dict

from checkov.common.util.env_vars_config import env_vars_config
from checkov.common.parallelizer.parallel_runner import parallel_runner
from checkov.terraform.module_loading.registry import module_loader_registry
from checkov.terraform.parser_utils import load_or_die_quietly

if TYPE_CHECKING:
    from checkov.terraform.module_loading.registry import ModuleLoaderRegistry


class ModuleDownload:
    def __init__(self, source_dir: str) -> None:
        self.source_dir = source_dir
        self.address: str | None = None
        self.module_name: str | None = None
        self.module_link: str | None = None
        self.tf_managed = False
        self.version: str | None = None

    def __str__(self) -> str:
        return f"{self.source_dir} -> {self.module_link} ({self.version})"


def find_tf_managed_modules(path: str) -> List[ModuleDownload]:
    """
    Leverage modules.json to better inform discovery. If we have this,
    there should be no need to walk and gather modules.
    """
    modules_found: list[ModuleDownload] = []

    tf_modules_file = Path(path) / '.terraform' / 'modules' / 'modules.json'
    if not tf_modules_file.exists():
        return modules_found

    for mod in json.loads(tf_modules_file.read_bytes())['Modules']:
        if mod['Key']:
            md = ModuleDownload(path)
            md.module_name = mod['Key']
            md.module_link = mod['Dir']
            md.version = mod['Version'] if 'Version' in mod else 'latest'
            md.address = f"{mod['Source']}:{md.version}"
            md.tf_managed = True
            modules_found.append(md)
    return modules_found


def find_modules(path: str, loaded_files_cache: Optional[Dict[str, Any]] = None,
                 parsing_errors: Optional[Dict[str, Exception]] = None, excluded_paths: Optional[list[str]] = None) -> list[ModuleDownload]:
    modules_found: list[ModuleDownload] = []
    if loaded_files_cache is None:
        loaded_files_cache = {}
    if parsing_errors is None:
        parsing_errors = {}

    excluded_paths_regex = re.compile('|'.join(f"({excluded_paths})")) if excluded_paths else None
    for root, _, full_file_names in os.walk(path):
        for file_name in full_file_names:
            if not file_name.endswith(".tf"):
                continue
            if root.startswith(os.path.join(path, ".terraform", "modules")):
                # don't scan the modules folder used by Terraform
                continue
            file_path = os.path.join(root, file_name)
            if excluded_paths_regex and excluded_paths_regex.search(file_path):
                continue

            data = load_or_die_quietly(file_path, parsing_errors)
            if not data:
                continue

            loaded_files_cache[file_path] = data
            if "module" not in data:
                continue
            for module in data["module"]:
                for module_name, module_data in module.items():
                    md = ModuleDownload(os.path.dirname(file_path))
                    md.module_name = module_name
                    md.module_link = module_data.get("source", [None])[0]
                    md.version = module_data.get("version", [None])[0]
                    if md.module_link:
                        md.address = f"{md.module_link}:{md.version}" if md.version else md.module_link
                    modules_found.append(md)
    return modules_found


def should_download(path: str | None) -> bool:
    return path is not None and not (path.startswith('./') or path.startswith('../') or path.startswith('/'))


def load_tf_modules(
    path: str,
    should_download_module: Callable[[str | None], bool] = should_download,
    run_parallel: bool = False,
    modules_to_load: List[ModuleDownload] | None = None,
    stop_on_failure: bool = False,
    loaded_files_cache: dict[str, Any] | None = None,
    parsing_errors: dict[str, Exception] | None = None,
    excluded_paths: List[str] | None = None,
) -> None:
    module_loader_registry.root_dir = path
    if not modules_to_load and env_vars_config.CHECKOV_EXPERIMENTAL_TERRAFORM_MANAGED_MODULES:
        modules_to_load = find_tf_managed_modules(path)
    if not modules_to_load:
        modules_to_load = find_modules(path, loaded_files_cache=loaded_files_cache, parsing_errors=parsing_errors, excluded_paths=excluded_paths)

    # To avoid duplicate work, we need to get the distinct module sources
    distinct_modules = list({m.address: m for m in modules_to_load}.values())

    downloadable_modules = [
        (module_loader_registry, m)
        for m in distinct_modules if should_download_module(m.module_link)
    ]

    if run_parallel:
        list(parallel_runner.run_function(_download_module, downloadable_modules))
    else:
        logging.info(f"Starting download of modules of length {len(downloadable_modules)}")
        for m in downloadable_modules:
            success = _download_module(*m)
            if not success and stop_on_failure:
                logging.info(f"Stopping downloading of modules due to failed attempt on {m[1].address}")
                break


def _download_module(ml_registry: ModuleLoaderRegistry, module_download: ModuleDownload) -> bool:
    logging.info(f'Downloading module {module_download.address}')
    try:
        content = ml_registry.load(
            current_dir=module_download.source_dir,
            source=module_download.module_link,
            source_version="latest" if not module_download.version else module_download.version,
            module_address=module_download.address,
            tf_managed=module_download.tf_managed,
        )
        if content is None or not content.loaded():
            if ml_registry.download_external_modules is not False:
                log_message = f'Failed to download module {module_download.address}'
                if ml_registry.download_external_modules is None:
                    log_message += ' (for external modules, the --download-external-modules flag is required)'
                logging.warning(log_message)
            return False
    except Exception as e:
        logging.warning(f"Unable to load module ({module_download.address}): {e}")
        return False

    return True



# DataCatalogWithPublicAccess
from typing import Dict, List, Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class DataCatalogWithPublicAccess(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure OCI Data Catalog is configured without overly permissive network access"
        id = "CKV_OCI_23"
        supported_resources = ['oci_datacatalog_catalog']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
        if "attached_catalog_private_endpoints" in conf:
            if len(conf["attached_catalog_private_endpoints"][0]) > 0:
                return CheckResult.PASSED
            else:
                return CheckResult.FAILED
        else:
            return CheckResult.FAILED

    def get_evaluated_keys(self) -> List[str]:
        return ['attached_catalog_private_endpoints']


check = DataCatalogWithPublicAccess()



from __future__ import annotations

from typing import Any, List

from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class IAMPassportAccountUsage(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure passport account is not used for assignment. Use service accounts and federated accounts where possible."
        id = "CKV_YC_24"
        categories = (CheckCategories.IAM,)
        supported_resources = (
            "yandex_resourcemanager_folder_iam_binding",
            "yandex_resourcemanager_folder_iam_member",
            "yandex_resourcemanager_cloud_iam_binding",
            "yandex_resourcemanager_cloud_iam_member",
            "yandex_organizationmanager_organization_iam_binding",
            "yandex_organizationmanager_organization_iam_member",
        )
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
        )

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        if self.entity_type == "yandex_resourcemanager_folder_iam_binding":
            for member in conf["members"][0]:
                if member.startswith("userAccount"):
                    return CheckResult.FAILED
            return CheckResult.PASSED
        if self.entity_type == "yandex_resourcemanager_folder_iam_member":
            if conf["member"][0].startswith("userAccount"):
                return CheckResult.FAILED
        if self.entity_type == "yandex_resourcemanager_cloud_iam_binding":
            for member in conf["members"][0]:
                if member.startswith("userAccount"):
                    return CheckResult.FAILED
            return CheckResult.PASSED
        if self.entity_type == "yandex_resourcemanager_cloud_iam_member":
            if conf["member"][0].startswith("userAccount"):
                return CheckResult.FAILED
        if self.entity_type == "yandex_organizationmanager_organization_iam_binding":
            for member in conf["members"][0]:
                if member.startswith("userAccount"):
                    return CheckResult.FAILED
            return CheckResult.PASSED
        if self.entity_type == "yandex_organizationmanager_organization_iam_member":
            if conf["member"][0].startswith("userAccount"):
                return CheckResult.FAILED
        return CheckResult.PASSED

    def get_evaluated_keys(self) -> List[str]:
        return ["members", "member"]


scanner = IAMPassportAccountUsage()



import ctypes
import subprocess  # nosec
import sys
from datetime import datetime
import json
import logging
import os
import platform
import re
import stat
from pathlib import Path
from typing import Optional, List, Set, Union, Dict, Any, Tuple, cast

from cachetools import cached, TTLCache
from pydantic import ValidationError

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.platform_integration import bc_integration
from checkov.common.bridgecrew.integration_features.features.policy_metadata_integration import integration as policy_metadata_integration
from checkov.common.bridgecrew.platform_key import bridgecrew_dir
from checkov.common.bridgecrew.severities import get_severity, Severity, Severities, BcSeverities
from checkov.common.models.enums import CheckResult
from checkov.common.output.report import Report
from checkov.common.sast.consts import CDKLanguages, SastLanguages
from checkov.common.sca.reachability.sast_contract.data_fetcher_sast_lib import SastReachabilityDataFetcher
from checkov.common.typing import _CheckResult
from checkov.common.util.http_utils import request_wrapper
from checkov.sast.checks_infra.base_registry import Registry
from checkov.sast.common import get_code_block_from_start
from checkov.sast.engines.base_engine import SastEngine
from checkov.sast.prisma_models.library_input import LibraryInput
from checkov.sast.prisma_models.policies_list import SastPolicies
from checkov.common.sast.consts import CDK_FRAMEWORK_PREFIX
from checkov.common.sast.report_types import PrismaReport, RuleMatch, create_empty_report
from checkov.sast.record import SastRecord
from checkov.sast.report import SastReport
from checkov.cdk.report import CDKReport
from checkov.sast.engines.files_filter_manager import FilesFilterManager

logger = logging.getLogger(__name__)

REPORT_PARSING_ERRORS = "report_parsing_errors"
FILE_NAME_PATTERN = re.compile(r"(\d+_\d+_\d+)_library\.(so|dll|dylib)")
SAST_CORE_FILENAME_PATTERN = re.compile(rf"{FILE_NAME_PATTERN.pattern}$")
SAST_CORE_URL_PATTERN = re.compile(rf".*/(?P<name>v?{FILE_NAME_PATTERN.pattern})\?.*")


class PrismaEngine(SastEngine):
    def __init__(self) -> None:
        self.lib_path = ""
        self.winmode = sys.platform.startswith('win')
        self.check_type = CheckType.SAST
        self.prisma_sast_dir_path = Path(bridgecrew_dir) / "sast"
        self.sast_platform_base_path = "api/v1/sast"
        self.enable_inline_suppressions = os.getenv("ENABLE_SAST_INLINE_SUPPRESSIONS", False)

    def get_check_thresholds(self, registry: Registry) -> Tuple[Severity, Severity]:
        """
        Returns a tuple of check threshold and skip check threshold..

        If a severity was specified in --check and / or --skip-check, then return a tuple of those values (these override enforcement rules).
        Else if enforcement rules are enabled, return a tuple of the enforcement rule's SAST soft fail threshold and NONE.
        Else return a tuple of NONE, NONE
        """
        none = Severities[BcSeverities.NONE]

        check_threshold: Optional[Severity] = registry.runner_filter.check_threshold  # type:ignore[union-attr] # not null
        skip_check_threshold: Optional[Severity] = registry.runner_filter.skip_check_threshold  # type:ignore[union-attr] # not null
        enforcement_threshold: Optional[Severity] = cast(Severity, registry.runner_filter.enforcement_rule_configs[self.check_type]) if registry.runner_filter.use_enforcement_rules else None  # type:ignore[union-attr] # not null

        return (check_threshold or none, skip_check_threshold or none) if (check_threshold or skip_check_threshold) else \
            (enforcement_threshold, none) if enforcement_threshold else \
            (none, none)

    def get_reports(self, targets: List[str], registry: Registry, languages: Set[SastLanguages], cdk_languages: List[CDKLanguages]) -> List[Report]:
        if not bc_integration.bc_api_key:
            logging.info("The --bc-api-key flag needs to be set to run SAST Prisma Cloud scanning")
            return []

        status = self.setup_sast_artifact()
        if not status:
            return []

        prisma_lib_path = self.get_sast_artifact()
        if not prisma_lib_path:
            return []

        self.lib_path = str(prisma_lib_path)

        check_threshold, skip_check_threshold = self.get_check_thresholds(registry)

        skip_paths = registry.runner_filter.excluded_paths if registry.runner_filter else []

        files_filter_manager = FilesFilterManager(targets, languages)
        skip_paths += files_filter_manager.get_files_to_filter()

        library_input: LibraryInput = {
            'languages': languages,
            'source_codes': targets,
            'policies': registry.checks_dirs_path,
            'checks': registry.runner_filter.checks if registry.runner_filter else [],
            'skip_checks': registry.runner_filter.skip_checks if registry.runner_filter else [],
            'check_threshold': check_threshold,
            'skip_check_threshold': skip_check_threshold,
            'platform_check_metadata': policy_metadata_integration.sast_check_metadata or {},
            'skip_path': skip_paths,
            'report_imports': registry.runner_filter.report_sast_imports if registry.runner_filter else False,
            'remove_default_policies': registry.runner_filter.remove_default_sast_policies if registry.runner_filter else False,
            'report_reachability': registry.runner_filter.report_sast_reachability if registry.runner_filter else False,
            'cdk_languages': cdk_languages
        }
        prisma_result = self.run_go_library(**library_input)

        return prisma_result

    def setup_sast_artifact(self) -> bool:
        current_version = ""
        if not self.prisma_sast_dir_path.exists():
            try:
                os.makedirs(self.prisma_sast_dir_path, exist_ok=True)
            except FileExistsError:
                pass
        else:
            is_file_exists = [f for f in os.listdir(self.prisma_sast_dir_path) if
                              (self.prisma_sast_dir_path / f).is_file() and "library" in f]
            if len(is_file_exists) > 0:
                latest_file = os.path.join(self.prisma_sast_dir_path, is_file_exists[0])
                creation_time = os.path.getmtime(latest_file)
                now = datetime.now().timestamp()
                diff = datetime.fromtimestamp(now) - datetime.fromtimestamp(creation_time)
                if diff.days < 1:
                    match = re.search(SAST_CORE_FILENAME_PATTERN, latest_file)
                    if match:
                        current_version = match.groups()[0]

        if os.getenv("SAST_ARTIFACT_PATH"):
            logging.debug(f'using local artifact in path {os.getenv("SAST_ARTIFACT_PATH")}')
            return True
        status: bool = self.download_sast_artifacts(current_version)

        return status

    @cached(TTLCache(maxsize=1, ttl=300))
    def download_sast_artifacts(self, current_version: str) -> bool:
        try:
            machine = get_machine()
            os_type = platform.system().lower()
            headers = bc_integration.get_default_headers("GET")
            headers["X-Client-Sast-Version"] = current_version
            headers["X-Required-Sast-Version"] = "latest"  # or ant version separated with _

            # don't use the 'should_call_raise_for_status' parameter for now, because it logs errors messages
            response = request_wrapper(
                method="GET",
                url=f"{bc_integration.api_url}/{self.sast_platform_base_path}/{os_type}/{machine}/artifacts",
                headers=headers,
            )
            response.raise_for_status()

            if response.status_code == 304:
                return True

            match = re.match(SAST_CORE_URL_PATTERN, response.url)
            if match:
                new_name = match.group('name')
                cli_file_name_path = self.prisma_sast_dir_path / new_name
                self._cleanup_scan()
                cli_file_name_path.touch(exist_ok=True)
                cli_file_name_path.write_bytes(response.content)
                cli_file_name_path.chmod(cli_file_name_path.stat().st_mode | stat.S_IEXEC | stat.S_IREAD)
                logging.debug("sast artifacts downloaded")
            return True
        except Exception:
            logging.debug(
                "Unexpected failure happened during downloading sast artifact. details are below.\n"
                "scanning is terminating. please try again. if it is repeated, please report.\n", exc_info=True)
            return False

    def _cleanup_scan(self) -> None:
        if self.prisma_sast_dir_path.exists():
            for file in os.scandir(self.prisma_sast_dir_path):
                os.unlink(file.path)
            # shutil.rmtree(self.prisma_sast_dir_path)
            logging.info('sast dir is clear')
        else:
            self.prisma_sast_dir_path.mkdir(exist_ok=True)

    def get_sast_artifact(self) -> Optional[Path]:
        env_variable_path = os.getenv("SAST_ARTIFACT_PATH")
        if env_variable_path and os.path.isfile(env_variable_path):
            return Path(env_variable_path)

        files = [(self.prisma_sast_dir_path / f) for f in os.listdir(self.prisma_sast_dir_path) if
                 (self.prisma_sast_dir_path / f).is_file() and "library" in f]

        if len(files) == 0:
            return None

        return files[0]

    def run_go_library(self, languages: Set[SastLanguages],
                       source_codes: List[str],
                       policies: List[str],
                       checks: List[str],
                       skip_checks: List[str],
                       skip_path: List[str],
                       check_threshold: Severity,
                       skip_check_threshold: Severity,
                       platform_check_metadata: Dict[str, Any],
                       cdk_languages: List[CDKLanguages],
                       list_policies: bool = False,
                       report_imports: bool = True,
                       report_reachability: bool = False,
                       remove_default_policies: bool = False) -> Union[List[Report], SastPolicies]:

        validate_params(languages, source_codes, list_policies)

        if bc_integration.bc_source:
            name = bc_integration.bc_source.name
        else:
            name = "unknown"

        reachability_data = None
        if report_reachability or report_imports:
            # TODO - run sast-core per src
            for source_code in source_codes:
                reachability_data = get_reachability_data(source_code)

        document = {
            "scan_code_params": {
                "source_codes": source_codes,
                "policies": policies,
                "languages": [a.value for a in languages],
                "checks": checks,
                "skip_checks": skip_checks,
                "skip_path": skip_path,
                "check_threshold": str(check_threshold),
                "skip_check_threshold": str(skip_check_threshold),
                "platform_check_metadata": platform_check_metadata,
                "list_policies": list_policies,
                "report_imports": report_imports,
                "remove_default_policies": remove_default_policies,
                "report_reachability": report_reachability,
                "reachability_data": reachability_data,
                "cdk_languages": [a.value for a in cdk_languages]
            },
            "auth": {
                "api_key": bc_integration.get_auth_token(),
                "platform_url": bc_integration.api_url,
                "client_name": name,
                "version": bc_integration.bc_source_version
            }
        }

        if list_policies:
            return self.run_go_library_list_policies(document)

        if self.winmode:
            sast_report = self._windows_sast_scan(document)
        else:
            sast_report = self._sast_default_scan(document)
        try:
            result = self.create_prisma_report(sast_report)
        except ValidationError as e:
            result = create_empty_report(list(languages))
            result.errors = {REPORT_PARSING_ERRORS: [str(err) for err in e.errors()]}
        return self.create_report(result)

    def _sast_default_scan(self, sast_input: Dict[str, Any]) -> Dict[str, Any]:
        library = ctypes.cdll.LoadLibrary(self.lib_path)
        analyze_code = library.analyzeCode
        analyze_code.restype = ctypes.c_void_p
        # send the document as a byte array of json format
        analyze_code_output = analyze_code(json.dumps(sast_input).encode('utf-8'))
        # we dereference the pointer to a byte array
        analyze_code_bytes = ctypes.string_at(analyze_code_output)
        # convert our byte array to a string
        analyze_code_string = analyze_code_bytes.decode('utf-8')
        return json.loads(analyze_code_string)  # type: ignore

    def _windows_sast_scan(self, sast_input: Dict[str, Any]) -> Dict[str, Any]:
        lib_dir_path = f"{os.path.dirname(self.lib_path)}"
        checkov_input_path = os.path.join(lib_dir_path, "checkov_input.json")
        sast_output_path = os.path.join(lib_dir_path, "sast_output.json")
        with open(checkov_input_path, 'w') as f:
            f.write(json.dumps(sast_input))
        log_level_str = "set LOG_LEVEL=" + os.getenv("LOG_LEVEL", "INFO")
        callargs = [log_level_str, "&", self.lib_path, checkov_input_path, sast_output_path]
        subprocess.run(callargs, shell=True)  # nosec B404, B603, B602

        with open(sast_output_path, 'r', encoding='utf-8') as f:
            report = f.read()
        parsed_report = json.loads(report)
        # cleanup
        os.remove(checkov_input_path)
        os.remove(sast_output_path)
        return parsed_report  # type: ignore

    def create_prisma_report(self, data: Dict[str, Any]) -> PrismaReport:
        if not data.get("imports"):
            data["imports"] = {}
        if not data.get("reachability_report"):
            data["reachability_report"] = {}

        self.remove_none_conf_incidents_policies(data)

        return PrismaReport(**data)

    @staticmethod
    def remove_none_conf_incidents_policies(data: Dict[str, Any]) -> None:
        remove_list = []  # type: ignore
        for lang, match in data.get('rule_match', dict()).items():
            for check in match.keys():
                if bc_integration.customer_run_config_response:
                    if check not in bc_integration.customer_run_config_response.get('policyMetadata', []):
                        remove_list.append((lang, check))

        for lang, check in remove_list:
            del data['rule_match'][lang][check]

    def run_go_library_list_policies(self, document: Dict[str, Any]) -> SastPolicies:
        try:
            library = ctypes.cdll.LoadLibrary(self.lib_path)
            list_policies = library.listPolicies
            list_policies.restype = ctypes.c_void_p

            # send the document as a byte array of json format
            list_policies_output = list_policies(json.dumps(document).encode('utf-8'))

            # we dereference the pointer to a byte array
            list_policies_bytes = ctypes.string_at(list_policies_output)

            # convert our byte array to a string
            list_policies_string = list_policies_bytes.decode('utf-8')

            d = json.loads(list_policies_string)
        except Exception as e:
            logging.error(e)
            return {}

        try:
            return SastPolicies(**d)
        except ValidationError:
            if d.get('errors'):
                logging.error(d.get('errors'))
            return {}

    def create_report(self, prisma_report: PrismaReport) -> List[Union[SastReport, CDKReport]]:
        logging.debug("Printing Prisma-SAST profiling data")
        logging.debug(prisma_report.profiler)
        reports: List[SastReport] = []
        for lang, checks in prisma_report.rule_match.items():
            sast_report = PrismaReport(rule_match={lang: checks}, errors=prisma_report.errors, profiler=prisma_report.profiler,
                                       run_metadata=prisma_report.run_metadata, imports={}, reachability_report={},
                                       skipped_checks_by_file=prisma_report.skipped_checks_by_file)
            report = SastReport(f'{self.check_type.lower()}_{lang.value}', prisma_report.run_metadata, lang, sast_report)
            for check_id, match_rule in checks.items():
                check_name = match_rule.check_name
                check_cwe = match_rule.check_cwe
                check_owasp = match_rule.check_owasp
                severity = get_severity(match_rule.severity)

                for match in match_rule.matches:
                    location = match.location
                    file_abs_path = location.path
                    file_path = file_abs_path.split('/')[-1]
                    file_line_range = [location.start.row, location.end.row]
                    split_code_block = [line + '\n' for line in location.code_block.split('\n')]
                    code_block = get_code_block_from_start(split_code_block, location.start.row)
                    metadata = match.metadata

                    if self.enable_inline_suppressions and any(skipped_check.check_id == match_rule.check_id for skipped_check in prisma_report.skipped_checks_by_file.get(file_abs_path, [])):
                        check_result = _CheckResult(
                            result=CheckResult.SKIPPED,
                            suppress_comment=next(skipped_check.suppress_comment for skipped_check in prisma_report.skipped_checks_by_file.get(file_abs_path, []) if skipped_check.check_id == match_rule.check_id))
                    else:
                        check_result = _CheckResult(result=CheckResult.FAILED)
                    record = SastRecord(check_id=check_id, check_name=check_name, resource="", evaluations={},
                                        check_class="", check_result=check_result, code_block=code_block,
                                        file_path=file_path, file_line_range=file_line_range, metadata=metadata,
                                        file_abs_path=file_abs_path, severity=severity, cwe=check_cwe,
                                        owasp=check_owasp, show_severity=True)
                    report.add_record(record)
            report_parsing_errors = prisma_report.errors.get(REPORT_PARSING_ERRORS)
            if report_parsing_errors:
                report.add_parsing_errors(report_parsing_errors)
            reports.append(report)

        for lang in prisma_report.imports:
            for report in reports:
                if report.language == lang:
                    report.sast_imports = prisma_report.imports[lang]
                    break
            else:
                sast_report = PrismaReport(rule_match={lang: {}}, errors=prisma_report.errors, profiler=prisma_report.profiler,
                                           run_metadata=prisma_report.run_metadata, imports={}, reachability_report={},
                                           skipped_checks_by_file={})
                report = SastReport(f'{self.check_type.lower()}_{lang.value}', prisma_report.run_metadata, lang, sast_report)
                report.sast_imports = prisma_report.imports[lang]
                reports.append(report)

        for lang in prisma_report.reachability_report:
            for report in reports:
                if report.language == lang:
                    report.sast_reachability = prisma_report.reachability_report[lang]
                    break
            else:
                sast_report = PrismaReport(rule_match={lang: {}}, errors=prisma_report.errors, profiler=prisma_report.profiler,
                                           run_metadata=prisma_report.run_metadata, imports={}, reachability_report={},
                                           skipped_checks_by_file={})
                report = SastReport(f'{self.check_type.lower()}_{lang.value}', prisma_report.run_metadata, lang, sast_report)
                report.sast_reachability = prisma_report.reachability_report[lang]
                reports.append(report)

        all_report = self._split_sast_cdk_reports(reports)
        return all_report

    def _split_sast_cdk_reports(self, sast_reports: List[SastReport]) -> List[Union[SastReport, CDKReport]]:
        cdk_reports: List[CDKReport] = []
        for report in sast_reports:
            for lang, rule_matches in report.sast_report.rule_match.items():
                sast_rule_matches: Dict[str, RuleMatch] = {}
                for policy_id, rule_match in rule_matches.items():
                    if rule_match.metadata.framework != CDK_FRAMEWORK_PREFIX:  # type: ignore
                        sast_rule_matches[policy_id] = rule_match
                        continue
                    self._update_cdk_report(lang, cdk_reports, report, policy_id, rule_match)

                if sast_rule_matches:
                    report.sast_report.rule_match[lang] = sast_rule_matches
                else:
                    report.sast_report.rule_match = {}
                self._update_sast_report_checks(report, cdk_reports)

        return self._get_all_reports(sast_reports, cdk_reports)

    @staticmethod
    def _update_cdk_report(lang: SastLanguages, cdk_reports: List[CDKReport], sast_report: SastReport, policy_id: str, rule_match: RuleMatch) -> None:
        if lang not in [c.language for c in cdk_reports]:
            new_cdk_report = PrismaReport(rule_match={lang: {}}, errors=sast_report.sast_report.errors,
                                          profiler=sast_report.sast_report.profiler,
                                          run_metadata=sast_report.sast_report.run_metadata,
                                          imports={}, reachability_report={}, skipped_checks_by_file={})
            new_report = CDKReport(f'{CDK_FRAMEWORK_PREFIX}_{lang.value}', sast_report.sast_report.run_metadata, lang, new_cdk_report)
            cdk_reports.append(new_report)
        for cdk_report in cdk_reports:
            if cdk_report.language == lang:
                cdk_report.cdk_report.rule_match[lang][policy_id] = rule_match
                for failed_check in sast_report.failed_checks:
                    if failed_check.check_id == policy_id:
                        cdk_report.failed_checks.append(failed_check)

                for skiped_check in sast_report.skipped_checks:
                    if skiped_check.check_id == policy_id:
                        cdk_report.skipped_checks.append(skiped_check)
                break

    def _update_sast_report_checks(self, report: SastReport, cdk_reports: List[CDKReport]) -> None:
        sast_failed_checks = []
        sast_skiped_checks = []

        if report.language not in [c.language for c in cdk_reports]:
            report.failed_checks = report.failed_checks
            report.skipped_checks = report.skipped_checks
            return

        for cdk_report in cdk_reports:
            fail_check = self._get_sast_check(report, cdk_report, report.failed_checks)
            if fail_check:
                sast_failed_checks.append(fail_check)
            skip_check = self._get_sast_check(report, cdk_report, report.skipped_checks)
            if skip_check:
                sast_skiped_checks.append(skip_check)

        report.failed_checks = sast_failed_checks
        report.skipped_checks = sast_skiped_checks

    @staticmethod
    def _get_sast_check(sast_report: SastReport, cdk_report: CDKReport, sast_report_checks: List[Any]) -> Any:
        for check in sast_report_checks:
            if sast_report.language == cdk_report.language and check.check_id not in [s.check_id for s in cdk_report.skipped_checks]:
                return check
        return None

    def _get_all_reports(self, sast_reports: List[SastReport], cdk_reports: List[CDKReport]) -> List[Union[SastReport, CDKReport]]:
        all_reports = []
        for report in sast_reports + cdk_reports:
            if report.check_type.startswith('cdk'):
                if report.cdk_report.rule_match:  # type: ignore
                    all_reports.append(report)
                    continue
            if report.check_type.startswith('sast'):
                if report.sast_report.rule_match or report.sast_reachability or report.sast_imports:  # type: ignore
                    all_reports.append(report)
                    continue
        return all_reports

    def get_policies(self, languages: Set[SastLanguages]) -> SastPolicies:
        if not bc_integration.bc_api_key:
            logging.info("The --bc-api-key flag needs to be set to run Sast prisma scanning")
            return []

        self.setup_sast_artifact()
        prisma_lib_path = self.get_sast_artifact()
        if not prisma_lib_path:
            return []

        self.lib_path = str(prisma_lib_path)

        library_input: LibraryInput = {
            'languages': languages,
            'list_policies': True,
            'source_codes': [],
            'policies': [],
            'checks': [],
            'skip_checks': [],
            'check_threshold': Severities[BcSeverities.NONE],
            'skip_check_threshold': Severities[BcSeverities.NONE],
            'platform_check_metadata': policy_metadata_integration.sast_check_metadata,
            'skip_path': [],
            'report_imports': False,
            'report_reachability': False,
            'cdk_languages': []
        }
        prisma_result = self.run_go_library(**library_input)
        return prisma_result


def validate_params(languages: Set[SastLanguages],
                    source_codes: List[str],
                    list_policies: bool) -> None:
    if list_policies:
        return

    if len(source_codes) == 0:
        raise Exception('must provide source code file or dir for sast runner')

    if len(languages) == 0:
        raise Exception('must provide a language for sast runner')


def get_machine() -> str:
    machine = platform.machine().lower()
    if machine in ['amd64', 'x86', 'x86_64', 'x64']:
        return "amd64"

    if machine in ['arm', 'arm64', 'armv8', 'aarch64', 'arm64-v8a']:
        return 'arm64'

    return ''


def get_reachability_data(repo_path: str) -> Dict[str, Any]:
    fetcher = SastReachabilityDataFetcher()
    reachability_data = fetcher.fetch(repository_name=repo_path, repository_root_dir=repo_path)
    data: Dict[str, Any] = {}
    if not reachability_data:
        return data
    langs = reachability_data.aliasMapping.get("languages")
    if not langs:
        return {}
    for lang, lang_data in langs.items():
        if lang == "nodejs":
            lang = "javascript"
        data[lang] = {"package_alias": {}}
        for _, files in lang_data.get("repositories", {}).items():
            for _, files_data in files.get("files", {}).items():
                for original_package_name, package_alias in files_data.get("packageAliases", {}).items():
                    aliases = package_alias.get("packageAliases", [])
                    if aliases:
                        data[lang]["package_alias"][original_package_name] = aliases[0]
    return data



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class ELBwListenerNotTLSSSL(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure AWS Elastic Load Balancer listener uses TLS/SSL"
        id = "CKV_AWS_376"
        supported_resource = ("aws_elb",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resource)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        if 'listener' in conf:
            self.evaluated_keys = ['listener']
            for listener_idx, listener in enumerate(conf.get('listener')):
                if 'instance_protocol' in listener:
                    self.evaluated_keys.append(f'listener/[{listener_idx}]/instance_protocol')
                    if listener.get('instance_protocol')[0].lower() in ('http', 'tcp'):
                        return CheckResult.FAILED
                    if listener.get('instance_protocol')[0].lower() in ('https', 'ssl') and \
                            ('ssl_certificate_id' not in listener or listener.get('ssl_certificate_id') == ""):
                        return CheckResult.FAILED

        return CheckResult.PASSED


check = ELBwListenerNotTLSSSL()



from typing import Dict, Any, List
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.util.oidc_utils import gh_abusable_claims, gh_repo_regex
import re
import logging


class GithubActionsOIDCTrustPolicy(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure GCP GitHub Actions OIDC trust policy is configured securely"
        id = "CKV_GCP_125"
        supported_resources = ["google_iam_workload_identity_pool_provider"]
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def extract_sub_claim_value(self, condition: str) -> str:
        """Extract the claim value from the condition string."""
        if not condition:
            return ""

        # Handle both single and double quotes
        claim_match = re.search(r"assertion\.sub\s*==\s*['\"]([^'\"]+)['\"]", condition)
        if claim_match:
            return claim_match.group(1)
        return ""

    def scan_resource_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        """Scans the configuration for GitHub Actions OIDC trust policy"""
        try:
            # Check issuer URI
            # If it's not OIDC or GitHub Actions URI, then pass
            issuer_oidc = conf.get("oidc")[0]
            if not issuer_oidc:
                return CheckResult.PASSED
            else:
                issuer_uri = issuer_oidc.get("issuer_uri", [None])[0]
                if not issuer_uri or issuer_uri != "https://token.actions.githubusercontent.com":
                    return CheckResult.PASSED

            # Check attribute mapping
            attribute_mapping = conf.get("attribute_mapping")
            if not attribute_mapping or not isinstance(attribute_mapping, list) or not attribute_mapping[0]:
                return CheckResult.FAILED
            attribute_mapping = attribute_mapping[0]
            if not attribute_mapping or "google.subject" not in attribute_mapping:
                return CheckResult.FAILED

            # Check attribute condition
            attribute_condition = conf.get("attribute_condition", False)[0]
            if not attribute_condition:
                return CheckResult.FAILED

            # Extract claim value
            sub_claim_value = self.extract_sub_claim_value(attribute_condition)
            if not sub_claim_value:
                return CheckResult.FAILED

            # If no colons - it means we assert something the value without the claim name, which is invalid when using GitHub Actions OIDC
            if ":" not in sub_claim_value:
                return CheckResult.FAILED

            # Break by colons; Since we already checked for the presence of colons, we can safely assume that the claim is in the form of claim_name:claim_value
            claim_parts = sub_claim_value.split(":")
            # Check if the first claim or value are wildcards - if yes, the assertion is checking nothing
            if claim_parts[0] == "*" or claim_parts[1] == "*":
                return CheckResult.FAILED

            # Check if the first claim is an abusable claim - if yes, the whole assertion can be abused
            if claim_parts[0] in gh_abusable_claims:
                return CheckResult.FAILED

            # Lastly, check for the classic "repo" claim
            if claim_parts[0] == "repo":
                # Check if the repo claim is in the form of org/repo
                if not gh_repo_regex.match(claim_parts[1]):
                    return CheckResult.FAILED

            return CheckResult.PASSED

        except Exception as e:
            logging.debug(f"CKV_GCP_125 failed with exception: {str(e)}")
            return CheckResult.FAILED

        def get_evaluated_keys(self) -> List[str]:
            return ["attribute_condition", "attribute_mapping", "issuer_uri"]


check = GithubActionsOIDCTrustPolicy()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class TwoFASignOnPolicyRule(BaseResourceCheck):
    def __init__(self):
        name = "Ensure 2FA is enabled for an Okta application signon policy rule"
        id = "CKV_OKTA_1"
        supported_resources = ["okta_app_signon_policy_rule"]
        categories = [CheckCategories.IAM]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> CheckResult:
        if "factor_mode" in conf:
            factor_mode = conf["factor_mode"][0]
            if factor_mode == "1FA":
                return CheckResult.FAILED

        return CheckResult.PASSED


check = TwoFASignOnPolicyRule()



from checkov.common.models.enums import CheckCategories
from checkov.arm.base_resource_value_check import BaseResourceValueCheck


class MySQLGeoBackupEnabled(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that My SQL server enables geo-redundant backups"
        id = "CKV_AZURE_94"
        supported_resources = ("Microsoft.DBforMySQL/flexibleServers",)
        categories = (CheckCategories.BACKUP_AND_RECOVERY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "properties/Backup/geoRedundantBackup"


check = MySQLGeoBackupEnabled()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import \
    BaseResourceValueCheck


class TKELogAgentEnable(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure Tencent Cloud TKE clusters enable log agent"
        id = "CKV_TC_6"
        supported_resources = ['tencentcloud_kubernetes_cluster']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "log_agent/enabled"

    def get_expected_value(self) -> bool:
        return True


check = TKELogAgentEnable()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class S3AccessPointPubliclyAccessible(BaseResourceCheck):

    def __init__(self):
        name = "Ensure AWS S3 access point block public access setting is enabled"
        id = "CKV_AWS_392"
        supported_resources = ['aws_s3_access_point']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'public_access_block_configuration' in conf:
            block_config_list = conf['public_access_block_configuration']
            if isinstance(block_config_list, list):
                block_config = block_config_list[0]
                if ('ignore_public_acls' in block_config and block_config['block_public_acls'] == [False] and
                        'block_public_policy' in block_config and block_config['block_public_policy'] == [False] and
                        'restrict_public_buckets' in block_config and
                        block_config['restrict_public_buckets'] == [False]):
                    self.evaluated_keys = ['public_access_block_configuration.block_public_acls',
                                           'public_access_block_configuration.restrict_public_buckets']
                    return CheckResult.FAILED
        return CheckResult.PASSED


check = S3AccessPointPubliclyAccessible()



from checkov.terraform.checks.resource.aws.AbsSecurityGroupUnrestrictedEgress import\
    AbsSecurityGroupUnrestrictedEgress


class SecurityGroupUnrestrictedEgressAll(AbsSecurityGroupUnrestrictedEgress):
    def __init__(self):
        super().__init__(check_id="CKV_AWS_382", port=-1)


check = SecurityGroupUnrestrictedEgressAll()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class StorageLocalUsers(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Avoid the use of local users for Azure Storage unless necessary"
        id = "CKV_AZURE_244"
        supported_resources = ('azurerm_storage_account',)
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict) -> CheckResult:
        # If local_user_enabled is explicitly True, return the default check result
        local_user_enabled = conf.get("local_user_enabled")
        if local_user_enabled is not None or local_user_enabled:
            return super().scan_resource_conf(conf)

        # Else only check if SFTP is enabled, which requires is_hns_enabled to exist and be True
        hns_enabled = conf.get("is_hns_enabled")
        if hns_enabled is None or not hns_enabled:
            return CheckResult.PASSED
        return super().scan_resource_conf(conf)

    def get_inspected_key(self) -> str:
        return 'local_user_enabled'

    def get_expected_value(self) -> bool:
        return False


check = StorageLocalUsers()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class RedshiftClusterWithCommonUsernameAndPublicAccess(BaseResourceCheck):

    def __init__(self):
        name = "Avoid AWS Redshift cluster with commonly used master username and public access setting enabled"
        id = "CKV_AWS_391"
        supported_resources = ['aws_redshift_cluster']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'master_username' in conf:
            if conf['master_username'][0] in ['awsuser', 'administrator', 'admin']:
                self.evaluated_keys = ['master_username']
                if 'publicly_accessible' in conf:
                    if str(conf['publicly_accessible'][0]).lower() == 'true':
                        return CheckResult.FAILED
                else:
                    return CheckResult.FAILED
        return CheckResult.PASSED


check = RedshiftClusterWithCommonUsernameAndPublicAccess()



from __future__ import annotations

from typing import Any

from cloudsplaining.scan.resource_policy_document import ResourcePolicyDocument

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class SNSCrossAccountAccess(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure AWS SNS topic policies do not allow cross-account access"
        id = "CKV_AWS_385"
        supported_resources = ("aws_sns_topic_policy",)
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        conf_policy = conf.get("policy")

        if not conf_policy:
            return CheckResult.PASSED

        if conf_policy:
            if isinstance(conf_policy[0], dict):
                for policy in conf_policy:
                    try:
                        processed_policy = ResourcePolicyDocument(policy=policy)
                        for statement in processed_policy.statements:
                            if statement.effect != "Allow":
                                continue

                            has_specific_aws_iam_arn_principal = False

                            aws_principal_values = []
                            if statement.statement and "Principal" in statement.statement and "AWS" in statement.statement["Principal"]:
                                raw_aws_principals = statement.statement["Principal"]["AWS"]
                                if isinstance(raw_aws_principals, str):
                                    aws_principal_values.append(raw_aws_principals)
                                elif isinstance(raw_aws_principals, list):
                                    aws_principal_values.extend(raw_aws_principals)

                            for principal_str in aws_principal_values:
                                if isinstance(principal_str, str) and \
                                        principal_str.startswith("arn:aws:iam::") and \
                                        principal_str != "*":
                                    has_specific_aws_iam_arn_principal = True
                                    break

                            if has_specific_aws_iam_arn_principal:
                                if not statement.conditions:
                                    return CheckResult.FAILED

                    except (TypeError, AttributeError):
                        return CheckResult.UNKNOWN
            else:
                return CheckResult.UNKNOWN
        return CheckResult.PASSED

    def get_evaluated_keys(self) -> list[str]:
        return ["policy"]


check = SNSCrossAccountAccess()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from typing import List


class ElasticacheReplicationGroupEncryptionAtTransitAuthToken(BaseResourceCheck):
    def __init__(self):
        name = "Ensure all data stored in the ElastiCache Replication Group is securely encrypted at transit and has auth token"
        id = "CKV_AWS_31"
        supported_resources = ['aws_elasticache_replication_group']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
            Looks for encryption configuration at aws_elasticache_replication_group:
            https://www.terraform.io/docs/providers/aws/r/elasticache_replication_group.html
        :param conf: aws_elasticache_replication_group configuration
        :return: <CheckResult>
        """
        if "transit_encryption_enabled" in conf.keys() and conf["transit_encryption_enabled"][0] \
                and ("auth_token" in conf.keys() or "user_group_ids" in conf.keys()):
            return CheckResult.PASSED
        return CheckResult.FAILED

    def get_evaluated_keys(self) -> List[str]:
        return ['transit_encryption_enabled', 'auth_token']


check = ElasticacheReplicationGroupEncryptionAtTransitAuthToken()
